<?php

/**
 * @file
 * Contains \Drupal\snippet_manager\Plugin\SnippetVariable\Entity.
 */

namespace Drupal\snippet_manager\Plugin\SnippetVariable;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\snippet_manager\SnippetVariableBase;
use Drupal\snippet_manager\SnippetVariableInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides Entity variable type.
 *
 * @SnippetVariable(
 *   id = "entity",
 *   title = @Translation("Entity"),
 *   category = @Translation("Entity"),
 *   deriver = "\Drupal\snippet_manager\Plugin\SnippetVariable\EntityDeriver",
 * )
 */
class Entity extends SnippetVariableBase implements SnippetVariableInterface, ContainerFactoryPluginInterface {

  /**
   * Snippet renderer.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityManager;

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Constructs entity variable object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_manager
   *   The entity manager service.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   *   The logger channel.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_manager, LoggerChannelInterface $logger) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityManager = $entity_manager;
    $this->entityType = $this->getDerivativeId();
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity.manager'),
      $container->get('logger.channel.snippet_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {

    $entity = NULL;
    if (isset($this->configuration['entity_id'])) {
      $storage = $this->entityManager->getStorage($this->entityType);
      $entity = $storage->load($this->configuration['entity_id']);

      if (!$entity) {
        drupal_set_message(
          t('Could not load entity: #%entity', ['%entity' => $this->configuration['entity_id']]),
          'error'
        );
      }
    }

    $form['entity_id'] = [
      '#type' => 'entity_autocomplete',
      '#title' => $this->entityType,
      '#default_value' => $entity,
      '#maxlength' => 2048,
      '#required' => TRUE,
      '#target_type' => $this->entityType,
    ];

    $default_view_mode = isset($this->configuration['view_mode']) ?
      $this->configuration['view_mode'] : 'default';

    $form['view_mode'] = [
      '#type' => 'select',
      '#options' => $this->entityManager->getViewModeOptions($this->getDerivativeId()),
      '#title' => t('View mode'),
      '#default_value' => $default_view_mode,
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $this->configuration['view_mode'] = $values['view_mode'];
    $this->configuration['entity_id'] = $values['entity_id'];
  }

  /**
   * {@inheritdoc}
   */
  public function getContent() {

    $build = [];

    $entity = $this->entityManager->getStorage($this->entityType)
      ->load($this->configuration['entity_id']);

    if ($entity) {
      $view_builder = $this->entityManager->getViewBuilder($this->entityType);
      $build = $view_builder->view($entity, $this->configuration['view_mode']);
    }
    else {
      $this->logger->error('Could not load entity: #%snippet', ['%entity' => $this->configuration['entity']]);
    }

    return $build;
  }

}
